@@ -13,7 +13,7 @@
 #include "CrashHandlerFixture.h"
 #include "gtest/gtest.h"
 #include "flang/Runtime/descriptor.h"
-#include "flang/Runtime/io-api.h"
+#include "flang/Runtime/io-api-consts.h"
 #include "flang/Runtime/main.h"
 #include "flang/Runtime/stop.h"
 #include "llvm/Support/raw_ostream.h"
@@ -43,11 +43,15 @@ TEST(ExternalIOTests, TestDirectUnformatted) {
   ASSERT_EQ(IONAME(EndIoStatement)(io), IostatOk)
       << "EndIoStatement() for OpenNewUnit";
 
+  StaticDescriptor<0> staticDescriptor;
+  Descriptor &desc{staticDescriptor.descriptor()};
+  desc.Establish(TypeCode{CFI_type_int8_t}, recl, &buffer, 0);
+  desc.Check();
+
   // INQUIRE(IOLENGTH=) j
   io = IONAME(BeginInquireIoLength)(__FILE__, __LINE__);
-  ASSERT_TRUE(IONAME(OutputUnformattedBlock)(
-      io, reinterpret_cast<const char *>(&buffer), recl, 1))
-      << "OutputUnformattedBlock() for InquireIoLength";
+  ASSERT_TRUE(IONAME(OutputDescriptor)(io, desc))
+      << "OutputDescriptor() for InquireIoLength";
   ASSERT_EQ(IONAME(GetIoLength)(io), recl) << "GetIoLength";
   ASSERT_EQ(IONAME(EndIoStatement)(io), IostatOk)
       << "EndIoStatement() for InquireIoLength";
@@ -59,24 +63,23 @@ TEST(ExternalIOTests, TestDirectUnformatted) {
     ASSERT_TRUE(IONAME(SetRec)(io, j)) << "SetRec(" << j << ')';
 
     buffer = j;
-    ASSERT_TRUE(IONAME(OutputUnformattedBlock)(
-        io, reinterpret_cast<const char *>(&buffer), 1, recl))
-        << "OutputUnformattedBlock()";
+    ASSERT_TRUE(IONAME(OutputDescriptor)(io, desc))
+        << "OutputDescriptor() for Write";
 
     ASSERT_EQ(IONAME(EndIoStatement)(io), IostatOk)
-        << "EndIoStatement() for OutputUnformattedBlock";
+        << "EndIoStatement() for Write";
   }
 
   for (int j{records}; j >= 1; --j) {
+    buffer = -1;
     // READ(UNIT=unit,REC=j) n
     io = IONAME(BeginUnformattedInput)(unit, __FILE__, __LINE__);
     ASSERT_TRUE(IONAME(SetRec)(io, j)) << "SetRec(" << j << ')';
-    ASSERT_TRUE(IONAME(InputUnformattedBlock)(
-        io, reinterpret_cast<char *>(&buffer), 1, recl))
-        << "InputUnformattedBlock()";
+    ASSERT_TRUE(IONAME(InputDescriptor)(io, desc))
+        << "InputDescriptor() for Read";
 
     ASSERT_EQ(IONAME(EndIoStatement)(io), IostatOk)
-        << "EndIoStatement() for InputUnformattedBlock";
+        << "EndIoStatement() for Read";
 
     ASSERT_EQ(buffer, j) << "Read back " << buffer
                          << " from direct unformatted record " << j
@@ -108,17 +111,21 @@ TEST(ExternalIOTests, TestDirectUnformattedSwapped) {
   ASSERT_EQ(IONAME(EndIoStatement)(io), IostatOk)
       << "EndIoStatement() for OpenNewUnit";
 
+  StaticDescriptor<0> staticDescriptor;
+  Descriptor &desc{staticDescriptor.descriptor()};
+  desc.Establish(TypeCode{CFI_type_int64_t}, recl, &buffer, 0);
+  desc.Check();
+
   static constexpr int records{10};
   for (int j{1}; j <= records; ++j) {
     // WRITE(UNIT=unit,REC=j) j
     io = IONAME(BeginUnformattedOutput)(unit, __FILE__, __LINE__);
     ASSERT_TRUE(IONAME(SetRec)(io, j)) << "SetRec(" << j << ')';
     buffer = j;
-    ASSERT_TRUE(IONAME(OutputUnformattedBlock)(
-        io, reinterpret_cast<const char *>(&buffer), recl, recl))
-        << "OutputUnformattedBlock()";
+    ASSERT_TRUE(IONAME(OutputDescriptor)(io, desc))
+        << "OutputDescriptor() for Write";
     ASSERT_EQ(IONAME(EndIoStatement)(io), IostatOk)
-        << "EndIoStatement() for OutputUnformattedBlock";
+        << "EndIoStatement() for Write";
   }
 
   // OPEN(UNIT=unit,STATUS='OLD',CONVERT='SWAP')
@@ -132,11 +139,10 @@ TEST(ExternalIOTests, TestDirectUnformattedSwapped) {
     // READ(UNIT=unit,REC=j) n
     io = IONAME(BeginUnformattedInput)(unit, __FILE__, __LINE__);
     ASSERT_TRUE(IONAME(SetRec)(io, j)) << "SetRec(" << j << ')';
-    ASSERT_TRUE(IONAME(InputUnformattedBlock)(
-        io, reinterpret_cast<char *>(&buffer), recl, recl))
-        << "InputUnformattedBlock()";
+    ASSERT_TRUE(IONAME(InputDescriptor)(io, desc))
+        << "InputDescriptor() for Read";
     ASSERT_EQ(IONAME(EndIoStatement)(io), IostatOk)
-        << "EndIoStatement() for InputUnformattedBlock";
+        << "EndIoStatement() for Read";
     ASSERT_EQ(buffer >> 56, j)
         << "Read back " << (buffer >> 56) << " from direct unformatted record "
         << j << ", expected " << j << '\n';
@@ -169,17 +175,6 @@ TEST(ExternalIOTests, TestSequentialFixedUnformatted) {
   ASSERT_EQ(IONAME(EndIoStatement)(io), IostatOk)
       << "EndIoStatement() for OpenNewUnit";
 
-  // INQUIRE(IOLENGTH=) j, ...
-  io = IONAME(BeginInquireIoLength)(__FILE__, __LINE__);
-  for (int j{1}; j <= 3; ++j) {
-    ASSERT_TRUE(IONAME(OutputUnformattedBlock)(
-        io, reinterpret_cast<const char *>(&buffer), recl, 1))
-        << "OutputUnformattedBlock() for InquireIoLength";
-  }
-  ASSERT_EQ(IONAME(GetIoLength)(io), 3 * recl) << "GetIoLength";
-  ASSERT_EQ(IONAME(EndIoStatement)(io), IostatOk)
-      << "EndIoStatement() for InquireIoLength";
-
   // INQUIRE(IOLENGTH=) j, ...
   StaticDescriptor<0> staticDescriptor;
   Descriptor &desc{staticDescriptor.descriptor()};
@@ -200,11 +195,10 @@ TEST(ExternalIOTests, TestSequentialFixedUnformatted) {
     // DO J=1,RECORDS; WRITE(UNIT=unit) j; END DO
     io = IONAME(BeginUnformattedOutput)(unit, __FILE__, __LINE__);
     buffer = j;
-    ASSERT_TRUE(IONAME(OutputUnformattedBlock)(
-        io, reinterpret_cast<const char *>(&buffer), recl, recl))
-        << "OutputUnformattedBlock()";
+    ASSERT_TRUE(IONAME(OutputDescriptor)(io, desc))
+        << "OutputDescriptor() for Write";
     ASSERT_EQ(IONAME(EndIoStatement)(io), IostatOk)
-        << "EndIoStatement() for OutputUnformattedBlock";
+        << "EndIoStatement() for WRITE";
   }
 
   // REWIND(UNIT=unit)
@@ -215,11 +209,10 @@ TEST(ExternalIOTests, TestSequentialFixedUnformatted) {
   for (int j{1}; j <= records; ++j) {
     // DO J=1,RECORDS; READ(UNIT=unit) n; check n; END DO
     io = IONAME(BeginUnformattedInput)(unit, __FILE__, __LINE__);
-    ASSERT_TRUE(IONAME(InputUnformattedBlock)(
-        io, reinterpret_cast<char *>(&buffer), recl, recl))
-        << "InputUnformattedBlock()";
+    ASSERT_TRUE(IONAME(InputDescriptor)(io, desc))
+        << "InputDescriptor() for Read";
     ASSERT_EQ(IONAME(EndIoStatement)(io), IostatOk)
-        << "EndIoStatement() for InputUnformattedBlock";
+        << "EndIoStatement() for Read";
     ASSERT_EQ(buffer, j) << "Read back " << buffer
                          << " from sequential fixed unformatted record " << j
                          << ", expected " << j << '\n';
@@ -232,11 +225,10 @@ TEST(ExternalIOTests, TestSequentialFixedUnformatted) {
         << "EndIoStatement() for Backspace (before read)";
     // READ(UNIT=unit) n
     io = IONAME(BeginUnformattedInput)(unit, __FILE__, __LINE__);
-    ASSERT_TRUE(IONAME(InputUnformattedBlock)(
-        io, reinterpret_cast<char *>(&buffer), recl, recl))
-        << "InputUnformattedBlock()";
+    ASSERT_TRUE(IONAME(InputDescriptor)(io, desc))
+        << "InputDescriptor() for Read";
     ASSERT_EQ(IONAME(EndIoStatement)(io), IostatOk)
-        << "EndIoStatement() for InputUnformattedBlock";
+        << "EndIoStatement() for Read";
     ASSERT_EQ(buffer, j) << "Read back " << buffer
                          << " from sequential fixed unformatted record " << j
                          << " after backspacing, expected " << j << '\n';
@@ -275,15 +267,18 @@ TEST(ExternalIOTests, TestSequentialVariableUnformatted) {
     buffer[j] = j;
   }
 
+  StaticDescriptor<0> staticDescriptor;
+  Descriptor &desc{staticDescriptor.descriptor()};
+
   for (int j{1}; j <= records; ++j) {
     // DO J=1,RECORDS; WRITE(UNIT=unit) BUFFER(0:j); END DO
     io = IONAME(BeginUnformattedOutput)(unit, __FILE__, __LINE__);
-    ASSERT_TRUE(IONAME(OutputUnformattedBlock)(io,
-        reinterpret_cast<const char *>(&buffer), j * sizeof *buffer,
-        sizeof *buffer))
-        << "OutputUnformattedBlock()";
+    desc.Establish(TypeCode{sizeof *buffer}, j * sizeof *buffer, buffer, 0);
+    desc.Check();
+    ASSERT_TRUE(IONAME(OutputDescriptor)(io, desc))
+        << "OutputDescriptor() for Write";
     ASSERT_EQ(IONAME(EndIoStatement)(io), IostatOk)
-        << "EndIoStatement() for OutputUnformattedBlock";
+        << "EndIoStatement() for Write";
   }
 
   // REWIND(UNIT=unit)
@@ -293,11 +288,12 @@ TEST(ExternalIOTests, TestSequentialVariableUnformatted) {
   for (int j{1}; j <= records; ++j) {
     // DO J=1,RECORDS; READ(UNIT=unit) n; check n; END DO
     io = IONAME(BeginUnformattedInput)(unit, __FILE__, __LINE__);
-    ASSERT_TRUE(IONAME(InputUnformattedBlock)(io,
-        reinterpret_cast<char *>(&buffer), j * sizeof *buffer, sizeof *buffer))
-        << "InputUnformattedBlock()";
+    desc.Establish(TypeCode{sizeof *buffer}, j * sizeof *buffer, buffer, 0);
+    desc.Check();
+    ASSERT_TRUE(IONAME(InputDescriptor)(io, desc))
+        << "InputDescriptor() for Read";
     ASSERT_EQ(IONAME(EndIoStatement)(io), IostatOk)
-        << "EndIoStatement() for InputUnformattedBlock";
+        << "EndIoStatement() for Read";
     for (int k{0}; k < j; ++k) {
       ASSERT_EQ(buffer[k], k) << "Read back [" << k << "]=" << buffer[k]
                               << " from direct unformatted record " << j
@@ -312,9 +308,9 @@ TEST(ExternalIOTests, TestSequentialVariableUnformatted) {
         << "EndIoStatement() for Backspace (before read)";
     // READ(unit=unit) n; check
     io = IONAME(BeginUnformattedInput)(unit, __FILE__, __LINE__);
-    ASSERT_TRUE(IONAME(InputUnformattedBlock)(io,
-        reinterpret_cast<char *>(&buffer), j * sizeof *buffer, sizeof *buffer))
-        << "InputUnformattedBlock()";
+    desc.Establish(TypeCode{sizeof *buffer}, j * sizeof *buffer, buffer, 0);
+    desc.Check();
+    ASSERT_TRUE(IONAME(InputDescriptor)(io, desc)) << "InputDescriptor()";
     ASSERT_EQ(IONAME(EndIoStatement)(io), IostatOk)
         << "EndIoStatement() for InputUnformattedBlock";
     for (int k{0}; k < j; ++k) {
@@ -935,10 +931,6 @@ TEST(ExternalIOTests, BigUnitNumbers) {
         static_cast<std::int64_t>(std::numeric_limits<ExternalUnit>::min()) - 1;
     EXPECT_EQ(IONAME(CheckUnitNumberInRange64)(unit64Ok, true), IostatOk);
     EXPECT_EQ(IONAME(CheckUnitNumberInRange64)(unit64Ok, false), IostatOk);
-    EXPECT_EQ(
-        IONAME(CheckUnitNumberInRange64)(unit64Bad, true), IostatUnitOverflow);
-    EXPECT_EQ(
-        IONAME(CheckUnitNumberInRange64)(unit64Bad2, true), IostatUnitOverflow);
     EXPECT_EQ(
         IONAME(CheckUnitNumberInRange64)(unit64Bad, true), IostatUnitOverflow);
     EXPECT_EQ(
@@ -949,7 +941,7 @@ TEST(ExternalIOTests, BigUnitNumbers) {
     std::snprintf(expectedMsg, n, "UNIT number %jd is out of range",
         static_cast<std::intmax_t>(unit64Bad));
     EXPECT_DEATH(
-        IONAME(CheckUnitNumberInRange64)(2147483648, false), expectedMsg);
+        IONAME(CheckUnitNumberInRange64)(unit64Bad, false), expectedMsg);
     for (auto i{std::strlen(expectedMsg)}; i < n; ++i) {
       expectedMsg[i] = ' ';
     }
